###
# Handles commands that need to be run.
###
class Orator.Libraries.CommandHandler

  ###
  # Constructs the command handler.  No arguments.
  ###
  constructor:->
    @commands = []

  ###
  # Lists all of the commands that are registered.
  ###
  all_commands:->

    for k of @commands
      k

  ###
  # Sets a command with the given name and function.
  ###
  command: (name, callback)->
    @commands[name] = callback

  ###
  # Runs a command in the context of inst with the arguments given.  The first
  # argument is the command, the second the context, and the rest of the
  # arguments are passed to the command.
  ###
  trigger: (command, inst, args...)->
    return false unless @commands[command]
    @commands[command].apply(inst, args)

  ###
  # Parses the line and runs the command given from the line in the context of
  # inst.  First argument is a string containing the command and arguments,
  # Second argument is the context to run under.
  ###
  run: (line, inst)->
    [command, args] = @parse(line)

    @trigger.apply(@, [command, inst].concat(args))

  ###
  # Parses a string containing the command and arguments.  Handles quotes, but
  # not escaping of them.
  #
  # @TODO: handle escaping quotes.
  # @TODO: handle escaping spaces.
  ###
  parse: (line)->
    command = new Orator.Libraries.StrBuf
    args = []
    state = { in_quote: false, argument: -1 }

    next_argument = ()->
      state.argument++
      args.push new Orator.Libraries.StrBuf

    append = (c)->
      if state.argument == -1
        command.append(c)
      else
        args.slice(-1)[0].append(c)


    for c in line

      switch c
        when ' '
          if state.in_quote
            append(c)
          else
            next_argument()
        when '"', "'"
          if state.in_quote == c
            state.in_quote = false
          else if state.in_quote isnt false
            append(c)
          else
            state.in_quote = c
        else append(c)

    [command.toString(), args.map (arg)-> arg.toString()]
