###
# Creates a buffer for a string.
###
class Orator.Libraries.StrBuf

  ###
  # Creates the buffer.  First argument is the string it starts with, the
  # second one is what is used when joining the strings together.
  ###
  constructor: (start = '', @join = '')->
    @parts = [start]
    @cache = null

  ###
  # Appends to the buffer.  Can take any number of arguments.  If the argument
  # is a StrBuf, concats its buffer to this buffer.  If it's anything else, it
  # tries its best to convert it to a string.
  ###
  append:->
    @cache = null

    for arg in arguments

      if arg._is_strbuf
        @parts = @parts.concat(arg.parts)
      else
        @parts.push((arg || "").toString())

  push: @prototype.append

  ###
  # Converts the buffer to a string.  Caches the result, if it can.
  ###
  toString:->
    @cache ||= @parts.join(@join)

  _is_strbuf: true

  to_s: @prototype.toString
